@@ -55,9 +55,13 @@ class Options:
         project_pull.add_argument(
             "--format",
             type=str,
-            default='yolov5',
             help="Output format"
         )
+        project_pull.add_argument(
+            "--owner",
+            type=str,
+            help="project owner"
+        )
 
         ## project push
         project_push = project_action.add_parser("push", help="Push images to project")
@@ -84,7 +88,7 @@ class Options:
             "--type",
             type=str,
             default='yolov5',
-            help="Model type, eg yolov5/yolov8/yolov8-seg"
+            help="Model type, eg yolov5/yolov8/yolov8-seg/yolov8-pose"
         )
 
         project_add_model.add_argument(
