@@ -6,6 +6,7 @@ from tqdm import tqdm
 from itertools import repeat
 import yaml
 import time
+import shutil
 
 class OCRResult:
     def __init__(self, client, data):
@@ -238,6 +239,43 @@ class MLModel:
     def add_version(self, weights, categories, architecture, params, metrics={}):
         return MLModelVersion.create(self.client, self.uuid, weights, categories, architecture, params, metrics=metrics)
 
+class Video:
+
+    def __init__(self, client, data):
+        self.client = client
+        self.data = data
+
+    @property
+    def uuid(self):
+        return self.data['uuid']
+
+    @property
+    def name(self):
+        return self.data['name']
+
+    @property
+    def video(self):
+        return self.data['video']
+    
+    @property
+    def created_at(self):
+        return self.data['created_at']
+    
+    @classmethod
+    def create(cls, client, project, video_file):
+        # create asset
+        asset = Asset.create(client, video_file)
+
+        # create sample
+        payload = {
+            'name': asset.filename,
+            'video': asset.uuid,
+        }
+        data = client.post(f'/projects/{project}/videos/', payload)
+
+        return cls(client, data)
+    
+
 class Sample:
 
     def __init__(self, client, data):
@@ -267,6 +305,17 @@ class Sample:
         }
         self.data['label'] = self.client.put(f'/samples/{self.uuid}/label/', payload=payload)
 
+    @property
+    def prediction(self):
+        return self.data['prediction']
+    
+    @prediction.setter
+    def prediction(self, annotations):
+        payload = {
+            'annotations': annotations
+        }
+        self.data['prediction'] = self.client.put(f'/samples/{self.uuid}/prediction/', payload=payload)
+
     @property
     def approved(self):
         return self.data['approved']
@@ -314,7 +363,8 @@ class Sample:
             urlretrieve(self.asset['url'], asset_location)
 
     def pull_label(self, location='./', type='BBOX', format=None):
-        if type=='BBOX':
+        if type in ['BBOX','KEYPOINT']:
+            format = format or 'yolov8'
             if format in ['yolov5', 'yolov8']:
                 asset_filename = self.asset['filename']
                 label_filename = os.path.splitext(asset_filename)[0] + '.txt'
@@ -340,12 +390,21 @@ class Sample:
                         xc, yc = xc / im_w, yc / im_h
                         w, h = w / im_w, h / im_h
 
-                        f.write(f'{cl} {xc:.5f} {yc:.5f} {w:.5f} {h:.5f}\n')
+                        f.write(f'{cl} {xc:.5f} {yc:.5f} {w:.5f} {h:.5f}')
 
+                        if type == 'KEYPOINT':
+                            keypoints = l['keypoints']
+                            for kx, ky, kv in zip(keypoints[0::3], keypoints[1::3], keypoints[2::3]):
+                                kx = min(max(0, kx), im_w) / im_w
+                                ky = min(max(0, ky), im_h) / im_h
+                                f.write(f' {kx:.5f} {ky:.5f} {kv}')
+
+                        f.write('\n')
             else:
                 raise Exception(f'Export format {format}" is not supported. Please check the documentation for the formats we support.')
         elif type=='INSTANCE_SEGMENTATION':
-            if format in ['yolov5', 'yolov8']:
+            format = format or 'yolov8'
+            if format in ['yolov8']:
                 from trainyolo.utils.yolov8 import annotations_to_yolo_polygons
                 
                 asset_filename = self.asset['filename']
@@ -363,7 +422,19 @@ class Sample:
                         for coord in polygon:
                             f.write(f' {coord:.5f}')
                         f.write('\n')
+            else:
+                raise Exception(f'Export format {format}" is not supported. Please check the documentation for the formats we support.')
+        elif type=='SEGMENTATION':
+            from trainyolo.utils.mask_utils import annotations_to_image
 
+            asset_filename = self.asset['filename']
+            label_filename = os.path.splitext(asset_filename)[0] + '.png'
+            label_location = os.path.join(location, 'labels', label_filename)
+
+            im_w, im_h = self.asset['metadata']['size'] 
+
+            _, cls = annotations_to_image(self.label['annotations'], (im_w, im_h))
+            cls.save(label_location)
         else:
             assert False, f'Type {type} is not supported.'
 
@@ -445,12 +516,12 @@ class Project:
 
     @classmethod
     def create(cls, client, name, categories=['object'], description='', annotation_type='BBOX'):
-        if annotation_type == 'BBOX':
+        if annotation_type in ['BBOX', 'KEYPOINT']:
             category_dict = [{'id': id + 1, 'name': name} for id, name in enumerate(categories)]
         elif annotation_type == 'INSTANCE_SEGMENTATION':
             category_dict = [{'id': id + 1, 'name': name, 'has_instances': True} for id, name in enumerate(categories)]
         else:
-            raise Exception(f'Annotation type "{annotation_type}" is not supported. Either use "BBOX" or "INSTANCE_SEGMENTATION".')
+            raise Exception(f'Annotation type "{annotation_type}" is not supported. Either use "BBOX" , "KEYPOINT" or "INSTANCE_SEGMENTATION".')
 
         payload = {
             'name': name,
@@ -486,7 +557,7 @@ class Project:
 
         return cls(client, data[0])
 
-    def pull(self, location='./', filter='LABELED', format=None):
+    def pull(self, location='./', filter='LABELED', format=None, trainval_dirs=False):
 
         print('Downloading project')
 
@@ -496,44 +567,55 @@ class Project:
         # load samples
         samples = self.get_samples(filter=filter)
 
-        # pull
+        # create output folders
         project_loc = os.path.join(location, self.name)
+        image_loc = os.path.join(project_loc, 'images')
+        label_loc = os.path.join(project_loc, 'labels')
 
-        if format in ['yolov5', 'yolov8']:
+        os.makedirs(image_loc, exist_ok=True)
+        os.makedirs(label_loc, exist_ok=True)
 
-            image_loc = os.path.join(project_loc, 'images')
-            label_loc = os.path.join(project_loc, 'labels')
+        print('Downloading samples... (this may take a while depending on your dataset size)')
+        with Pool(8) as p:
+            inputs = zip(samples, repeat(project_loc), repeat(self.annotation_type), repeat(format))
+            r = p.starmap(Sample.pull, tqdm(inputs, total=len(samples)))
 
-            os.makedirs(image_loc, exist_ok=True)
-            os.makedirs(label_loc, exist_ok=True)
+        # create train-val txt file
+        with open(os.path.join(project_loc, 'train.txt'), 'w') as f_train, open(os.path.join(project_loc, 'val.txt'), 'w') as f_val:
+            for s in samples:
+                if s.split == 'TRAIN':
+                    f_train.write(f'./images/{s.asset["filename"]}\n')
+                if s.split == 'VAL':
+                    f_val.write(f'./images/{s.asset["filename"]}\n')
 
-            print('Downloading samples... (this may take a while depending on your dataset size)')
-            with Pool(8) as p:
-                inputs = zip(samples, repeat(project_loc), repeat(self.annotation_type), repeat('yolov5'))
-                r = p.starmap(Sample.pull, tqdm(inputs, total=len(samples)))
+        # create dataset yaml file
+        with open(os.path.join(project_loc, 'dataset.yaml'), 'w') as f:
+            yaml_content = {
+                'path': os.path.abspath(project_loc),
+                'train': 'train.txt',
+                'val': 'val.txt',
+                'names': {cat['id']-1:cat['name'] for cat in self.categories}
+            }
+            if self.annotation_type == 'KEYPOINT':
+                yaml_content['kpt_shape'] = [len(self.categories[0]['keypoints']),3]
+            yaml.dump(yaml_content, f, default_flow_style=False)
 
-            # create train-val txt file
-            with open(os.path.join(project_loc, 'train.txt'), 'w') as f_train, open(os.path.join(project_loc, 'val.txt'), 'w') as f_val:
-                for s in samples:
-                    if s.split == 'TRAIN':
-                        f_train.write(f'./images/{s.asset["filename"]}\n')
-                    if s.split == 'VAL':
-                        f_val.write(f'./images/{s.asset["filename"]}\n')
+        # create seperate train/val folder with symlinks
+        if trainval_dirs:
+            
+            # remove folders as train/val spit might have changed
+            [shutil.rmtree(os.path.join(project_loc, split, f)) for split in ['train', 'val'] for f in ['images', 'labels'] if os.path.exists(os.path.join(project_loc, split, f))]
 
-            # create dataset yaml file
-            with open(os.path.join(project_loc, 'dataset.yaml'), 'w') as f:
-                yaml_content = {
-                    'path': os.path.abspath(project_loc),
-                    'train': 'train.txt',
-                    'val': 'val.txt',
-                    'names': {cat['id']-1:cat['name'] for cat in self.categories}
-                }
-                yaml.dump(yaml_content, f)
+            # create folders
+            [os.makedirs(os.path.join(project_loc, split, f), exist_ok=True) for split in ['train', 'val'] for f in ['images', 'labels']]
 
-            return project_loc
-
-        else:
-            raise Exception(f'Export format "{format}" is not available. Please check our documentation for the different formats we support.')
+            # create symlinks
+            for s in samples:
+                txt_filename = s.asset["filename"].rsplit('.',1)[0] + '.txt'
+                os.symlink(f'../../images/{s.asset["filename"]}', f'{os.path.abspath(project_loc)}/{"train" if s.split == "TRAIN" else "val"}/images/{s.asset["filename"]}')
+                os.symlink(f'../../labels/{txt_filename}', f'{os.path.abspath(project_loc)}/{"train" if s.split == "TRAIN" else "val"}/labels/{txt_filename}')
+        
+        return project_loc
 
 
     def push(self, image_list):
@@ -543,9 +625,12 @@ class Project:
 
     def get_samples(self, filter='LABELED'):
 
-        query = ''
-        if 'labeled' in filter.lower():
-            query += '&labeled=True'
+        if filter.lower() == 'labeled':
+            query = 'labeled=True'
+        elif filter.lower() == 'unlabeled':
+            query = 'labeled=False'
+        else:
+            query = ''
 
         sample_list = self.client.get(f'/projects/{self.uuid}/sample_list/?{query}')
         
